'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.processImport = processImport;
exports.processKeyWordReplace = processKeyWordReplace;

exports.default = function (ast, options) {
    let scopeManager = (0, _escope.analyze)(ast, {
        sourceType: _parseOptions2.default.sourceType,
        ecmaVersion: _parseOptions2.default.ecmaFeatures.ecmaVersion
    });
    let getDef = (0, _util.defAnalyze)(ast);

    let importPaths = processImport(ast, options);
    // 查找export default {}
    let exportObject = getExportObject(ast, scopeManager);
    processKeyWordReplace(ast);
    return {
        exportObject,
        importPaths,
        data: processData(exportObject),
        computed: processComputed(exportObject),
        components: processComponents(exportObject, options, getDef),
        methods: processMethods(exportObject),
        props: processProps(exportObject)
    };
};

var _esquery = require('esquery');

var _esquery2 = _interopRequireDefault(_esquery);

var _esrecurse = require('esrecurse');

var _esrecurse2 = _interopRequireDefault(_esrecurse);

var _escope = require('escope');

var _util = require('../util');

var _utils = require('../../utils');

var _config = require('../../config');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _parseOptions = require('../parseOptions');

var _parseOptions2 = _interopRequireDefault(_parseOptions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getExportObject(ast, scope) {
    let exportObject = (0, _esquery2.default)(ast, 'ExportDefaultDeclaration')[0];
    if (!exportObject) {
        // 没有export
        throw 'export not found';
    }
    exportObject = exportObject.declaration;
    if (exportObject.type !== 'ObjectExpression') {
        // 其他export形式，
        // 比如 let a = {}; export default a;
        if (exportObject.type === 'Identifier') {
            let ref = scope.resolve(exportObject);
            if (ref) {
                exportObject = ref.resolved.defs[0].node.init;
            } else {
                throw 'export form unknown';
            }
        } else {
            throw 'export form unknown';
        }
    }
    return exportObject;
}

function processComputed(ast) {
    // 找到所有computed
    let computed = (0, _esquery2.default)(ast, 'ObjectExpression>Property[key.name="computed"]>ObjectExpression>Property');
    // 当前只处理get的computed，后续需要判断是否是可以set的，
    // 复制一份ast，分别生成set和get
    computed = computed.map(function (item) {
        // 从 property 修改为 MethodDefinition
        item.type = 'MethodDefinition';
        // 从 init 修改为get
        item.kind = 'get';
        return item;
    });
    return computed;
}

function processMethods(ast) {
    // 找到所有 method
    let methods = (0, _esquery2.default)(ast, 'ObjectExpression>Property[key.name="methods"]>ObjectExpression>Property');
    // 当前只处理get的computed，后续需要判断是否是可以set的，
    // 复制一份ast，分别生成set和get
    methods = methods.map(function (item) {
        // 从 property 修改为 MethodDefinition
        item.type = 'MethodDefinition';
        // 从 init 修改为get
        item.kind = 'method';
        return item;
    });
    return methods;
}

function processComponents(ast, options, getDef) {
    let dir = (0, _util.getPackageInfo)(options.filePath).dir;
    // 找到所有 components
    let components = (0, _esquery2.default)(ast, 'ObjectExpression>Property[key.name="components"]')[0];
    if (!components) {
        return;
    }
    // 把 type 从 property 改为 classproperty
    // 按照ast标准本应是ClassProperty，但是espree不认，只好用使用原始的property，到php-generator里再判断是否是类属性，然后生成不一样的代码
    // components.type = 'ClassProperty';
    // 找到所有依赖的组件，把变量引用替换成路径
    // 本来按照js语法是可以直接传递class引用的，生成代码都ok了，一切都非常美好，
    // 但是作为世界上最好的语言，php没法传递class引用，只好传递class名字
    (0, _esquery2.default)(components, 'Property>ObjectExpression>Property').map(function (component) {
        let def = getDef(component.value);
        let importPath = _path2.default.resolve(dir, def.parent.source.value);
        let { useNamespaceConverted } = (0, _util.getPackageInfo)(importPath);
        // console.log('>>><<<', getPackageInfo(importPath));
        let namespace = useNamespaceConverted.replace(/\\/g, '\\\\');
        component.key.name = component.key.name.replace(/-/g, '').toLowerCase();
        // 把变量引用换成字符串
        component.value = {
            type: 'Literal',
            value: namespace,
            raw: '"' + namespace + '"'
        };
    });
    return components;
}

function processData(ast) {
    let data = (0, _esquery2.default)(ast, 'ObjectExpression>Property[key.name="data"]')[0];
    if (!data) {
        return;
    }
    // 从 property 修改为 MethodDefinition
    data.type = 'MethodDefinition';
    // 从 init 修改为get
    data.kind = 'method';
    return data;
}

/**
 * 修改props的ast，props有很多形式1、数组, 2、对象xxx: String, 3、对象xxx: {type: String}
 * 最后都转化为对象xxx: {default: xxx}
 * 只保留default属性，其他的类型检测不在php里做，php只管最稳定的输出首屏dom
 * 至于有些代码写的传入类型和props不匹配，会在浏览器console里提示的，
 * 开发者自己去改就好了，在首屏渲染就不矫情这些事了
 *
 * @param {AST} ast export的ast
 *
 * @return {AST=}
 */
function processProps(ast) {
    // 数组形式的props
    let props = (0, _esquery2.default)(ast, 'ObjectExpression>Property[key.name="props"]')[0];
    if (!props) {
        return;
    }

    if (props.value.type === 'ObjectExpression') {
        // 删除default以外所有属性
        props.value.properties.map(function (node) {
            if (node.value.type === 'ObjectExpression') {
                // 只保留default字段
                node.value.properties = node.value.properties.filter(function (property) {
                    return property.key.name === 'default';
                });
            } else {
                // node.value一定是一个object
                // 避免运行时判断这判断那的，烦
                node.value = {
                    type: 'ObjectExpression',
                    properties: []
                };
            }
        });
        return props;
    }

    let elements = [];
    if (props.value.type === 'ArrayExpression') {
        // 把数组转换成对象
        // props: ['a', 'b', 'c']
        // 转换成
        // props: {
        //  a: {},
        //  b: {},
        //  c: {}
        // }
        props.value.elements.map(function (node) {
            elements.push({
                type: 'Property',
                method: false,
                shorthand: false,
                computed: false,
                key: {
                    type: 'Identifier',
                    name: node.value
                },
                value: {
                    type: 'ObjectExpression',
                    properties: []
                },
                kind: 'init'
            });
        });

        props.value = {
            type: 'ObjectExpression',
            properties: []
        };

        props.value.properties = props.value.properties.concat(elements);

        return props;
    }

    return;
}

function processImport(ast, options) {
    let dir = (0, _util.getPackageInfo)(options.filePath).dir;

    let imports = (0, _esquery2.default)(ast, 'ImportDeclaration');
    return imports.map(function (item) {
        let importPath;
        if (/^[./\\]/.test(item.source.value)) {
            // 是相对路径
            // 先把import a from '../a';中的../a转换为绝对路径
            importPath = _path2.default.resolve(dir, item.source.value);
        } else {
            // 绝对路径，从packages路径加载第三方包
            if (/[/\\]/.test(item.source.value)) {
                // 如果包名是xxx/yyy
                importPath = _path2.default.resolve(_config.packagePath, item.source.value);
            } else {
                // 包名仅仅是xxx
                importPath = _path2.default.resolve(_config.packagePath, item.source.value, 'index');
            }
        }

        // 根据路径查找磁盘上存在的文件，
        // 比如，import xxx from './xxx', 
        // 查找./xxx是否存在,不存在就继续查找./xxx/index等
        let filePathInfo = (0, _utils.getFilePath)(importPath);

        if (filePathInfo) {
            importPath = filePathInfo.filePathWithOutExt;
        }
        let originalPath = item.source.value;

        item.source.value = _path2.default.relative(dir, importPath);
        let { namespace, useNamespaceConverted } = (0, _util.getPackageInfo)(importPath);
        // 最后把修改import后路径保存在namespace字段
        // 如果找到的是php就不增加命名空间
        // 毕竟人家写的代码，也不知道用没用命名空间
        let filePath = '';
        if (filePathInfo) {
            filePath = filePathInfo.filePath;
        }
        if (/php$/.test(filePath)) {
            if (/^\./.test(originalPath)) {
                // 原始路径是相对路径，比如./util.php，则需要转成命名空间
                item.namespace = {
                    type: 'Literal',
                    value: namespace
                };
            } else {
                // 原始路径是绝对路径，比如Smarty， 就不需要转成命名空间，用原名就行了
                // 因为单独的包形式php一般是公用库，是不使用命名空间的
                // 如果namespace给null会导致没有命名空间生命，在当前命名空间下找不到这个类，就挂了，所以还是要namespace的
                item.namespace = {
                    type: 'Literal',
                    value: originalPath
                };
            }
            // item.namespace = null;
        } else {
            item.namespace = {
                type: 'Literal',
                value: namespace
            };
            // 处理default export
            item.specifiers.map(function (specifier) {
                if (specifier.type === 'ImportDefaultSpecifier') {
                    specifier.raw = _config.defaultExportName;
                }
            });
        }
        return importPath;
    })
    // 筛选出非空子集，就是所有的相对路径模块
    .filter(item => !!item);
}

function processKeyWordReplace(ast) {
    let keyWordReplace = _config.customConfig.keyWordReplace || {};
    _esrecurse2.default.visit(ast, {
        Identifier: function (node) {
            Object.keys(keyWordReplace).map(key => {
                if (node.name === key) {
                    node.name = keyWordReplace[key];
                }
            });
        }
    });
}