'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.recursiveCompile = recursiveCompile;
exports.compile = compile;
exports.compileSFCFile = compileSFCFile;
exports.compileJsFile = compileJsFile;
exports.compileSFC = compileSFC;

var _index = require('./ast/index');

var _util = require('./ast/util');

var _index2 = require('./ast/template/index');

var _index3 = _interopRequireDefault(_index2);

var _index4 = require('./ast/script/index');

var _index5 = _interopRequireDefault(_index4);

var _config = require('./config');

var _genClass = require('./ast/genClass');

var _utils = require('./utils');

var _resolveTemplateResource = require('./resolveTemplateResource');

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _mkdirp = require('mkdirp');

var _mkdirp2 = _interopRequireDefault(_mkdirp);

var _shelljs = require('shelljs');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let vueTemplateCompiler = require('vue-template-compiler');
let phpGenerator = require('php-generator');


let baseName = 'base';

let importBaseStr = `
    import {
        add as func_add,
        arr as func_arr,
        getArr as func_getArr
    } from '${baseName}';
`;

function ifNeedProcess(filePath) {
    return (/\.js$|\.vue$|\.jsx$|\.es6$/.test(filePath)
    );
}

function copyToOutput(filePath) {
    let { outputFilePath } = (0, _utils.getOutputFilePath)(filePath);
    return (0, _shelljs.cp)(filePath, outputFilePath);
}

async function recursiveCompile(filePath) {
    let compiledPath = {};
    async function rCompile(filePath) {
        let filePathInfo = (0, _utils.getFilePath)(filePath);

        if (!filePathInfo) {
            return;
        }

        let existsPath = filePathInfo.filePath;

        if (compiledPath[existsPath]) {
            return;
        }

        compiledPath[existsPath] = true;

        if (!ifNeedProcess(existsPath)) {
            copyToOutput(existsPath);
            return;
        }

        console.log('compiling file: ', existsPath);

        let { importPaths } = await compile(existsPath);
        let promises = importPaths.map(async function (importPath) {
            return await rCompile(importPath);
        });

        return await Promise.all(promises);
    }
    if (Array.isArray(filePath)) {
        return Promise.all(filePath.map(async filePath => await rCompile(filePath)));
    } else {
        return await rCompile(filePath);
    }
}

async function compile(filePath) {
    let { ext } = _path2.default.parse(filePath);
    if (ext === '.vue') {
        return compileSFCFile(filePath);
    } else if (ext === '.js') {
        return compileJsFile(filePath);
    } else {
        throw 'file type not support: ' + filePath;
    }
}

async function compileSFCFile(filePath) {
    return new Promise(function (resolve, reject) {
        // 读取文件
        _fs2.default.readFile(filePath, async function (err, data) {
            if (err) {
                reject(err);
                return;
            }
            let {
                dir,
                name
            } = (0, _util.getPackageInfo)(filePath);
            let ret = await compileSFC(data.toString(), { filePath });
            // 把生成的代码写入文件
            let phpPath = _path2.default.resolve(dir, name + '.php');
            let {
                outputFileDir,
                outputFilePath
            } = (0, _utils.getOutputFilePath)(phpPath);
            (0, _mkdirp2.default)(outputFileDir, function () {

                _fs2.default.writeFile(outputFilePath, ret.phpCode, function (err) {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve(ret);
                });
            });
        });
    });
}

async function compileJsFile(filePath) {
    return new Promise(function (resolve, reject) {
        // 读取文件
        _fs2.default.readFile(filePath, function (err, data) {
            if (err) {
                reject(err);
                return;
            }
            let {
                dir,
                namespace,
                namespaceConverted,
                name
            } = (0, _util.getPackageInfo)(filePath);

            let content = data;
            // base文件本身就不import base了
            if (!/base\.index/.test(namespace)) {
                content = importBaseStr + data;
            }
            let ast = (0, _index.parse)(content);
            (0, _util.defaultExport2NamedExport)(ast, filePath);
            let importPaths = (0, _index4.processImport)(ast, { filePath });
            (0, _index4.processKeyWordReplace)(ast);
            (0, _util.addNamespace)(ast, namespace, namespaceConverted);
            let phpCode = phpGenerator.generate(ast);
            // 把生成的代码写入文件
            let phpPath = _path2.default.resolve(dir, name + '.php');

            let {
                outputFileDir,
                outputFilePath
            } = (0, _utils.getOutputFilePath)(phpPath);

            (0, _mkdirp2.default)(outputFileDir, function () {
                _fs2.default.writeFile(outputFilePath, phpCode, function (err) {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve({
                        importPaths,
                        phpCode
                    });
                });
            });
        });
    });
}

async function compileSFC(vueContent, options = {}) {
    let filePath = options.filePath;
    let {
        dir,
        namespace,
        namespaceConverted
    } = (0, _util.getPackageInfo)(filePath);

    let vueName = 'vue';
    // 生成class的ast
    let classAst = (0, _genClass.genClass)(_config.defaultExportName, vueName);

    // 把vue文件拆分成template、script、style几个模块
    // style 内容不处理了，使用webpack extract出来的css，省心
    // 只要得到style的scopeId就可以
    let { template, script, styles } = vueTemplateCompiler.parseComponent(vueContent);

    let hasScoped = styles.some(style => style.scoped);
    let scopeId = hasScoped ? (0, _utils.getScopeId)(filePath, vueContent) : '';
    // 用ssrCompile对template模块进行处理
    let vdom = vueTemplateCompiler.ssrCompile(template.content, {
        scopeId: scopeId,
        modules: [(0, _resolveTemplateResource.markResource)()]
    });
    // console.log(vdom);

    let templateCode = await (0, _resolveTemplateResource.resolveTemplateResource)(`
        function _render(_ssrRenderData=[]) {
            ${vdom.render}
        }
    `, { dir });

    // 对ssrCompile生成的代码进行ast解析
    let templateAst = (0, _index.parse)(templateCode, {
        sourceType: 'script',
        tokens: false,
        ecmaVersion: 9
    });

    // 改造template的ast
    templateAst = (0, _index3.default)(templateAst, { scopeId });
    (0, _genClass.addMethod)(classAst, templateAst);

    let vueReg = /import\svue/i;
    let content = '';
    if (vueReg.test(script.content)) {
        content = `
            ${importBaseStr}
            ${script.content}
        `;
    } else {
        content = `
            ${importBaseStr}
            import ${vueName} from '${vueName}';
            ${script.content}
        `;
    }
    let scriptAst = (0, _index.parse)(content);
    let {
        components,
        computed,
        data,
        exportObject,
        importPaths,
        methods,
        props
    } = (0, _index5.default)(scriptAst, options);
    (0, _genClass.addMethods)(classAst, computed);
    (0, _genClass.addMethods)(classAst, methods);
    (0, _genClass.addProperty)(classAst, components);
    (0, _genClass.addProperty)(classAst, data);
    (0, _genClass.addProperty)(classAst, props);
    (0, _genClass.addProperty)(classAst, {
        type: 'Property',
        key: {
            type: 'Identifier',
            name: 'hasScoped'
        },
        value: {
            type: 'Literal',
            value: hasScoped,
            raw: '' + hasScoped
        }
    });
    (0, _genClass.addProperty)(classAst, {
        type: 'Property',
        key: {
            type: 'Identifier',
            name: 'scopeId'
        },
        value: {
            type: 'Literal',
            value: scopeId,
            raw: "'" + scopeId + "'"
        }
    });

    scriptAst = (0, _index.replace)(scriptAst, exportObject, classAst);
    (0, _util.addNamespace)(scriptAst, namespace, namespaceConverted);

    // 把改造完的ast生成php code
    // let phpCode = phpGenerator.generate(templateAst);
    // let phpCode = phpGenerator.generate(classAst);
    let phpCode = phpGenerator.generate(scriptAst);

    return {
        vdom,
        importPaths,
        phpCode
    };
}